# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"create_session": "00_core.ipynb",
         "create_cached_session": "00_core.ipynb",
         "get_max_workers": "00_core.ipynb",
         "load_url_image": "00_core.ipynb",
         "save_image": "00_core.ipynb",
         "download_image": "00_core.ipynb",
         "parse_box": "00_core.ipynb",
         "create_iiif_url": "00_core.ipynb",
         "iiif_df_apply": "00_core.ipynb",
         "image_extensions": "00_core.ipynb",
         "bytesto": "00_core.ipynb",
         "get_json_url": "01_sample.ipynb",
         "load_json": "01_sample.ipynb",
         "count_json_iter": "01_sample.ipynb",
         "get_year_size": "01_sample.ipynb",
         "get_year_sizes": "01_sample.ipynb",
         "get_all_year_sizes": "01_sample.ipynb",
         "sample_stream": "01_sample.ipynb",
         "calc_frac_size": "01_sample.ipynb",
         "calc_year_from_total": "01_sample.ipynb",
         "reduce_df_memory": "01_sample.ipynb",
         "nnSampler": "01_sample.ipynb",
         "sample_year": "01_sample.ipynb",
         "create_sample": "01_sample.ipynb",
         "download_sample": "01_sample.ipynb",
         "create_label_studio_json": "02_annotate.ipynb",
         "process_labels": "02_annotate.ipynb",
         "load_annotations_csv": "02_annotate.ipynb",
         "check_download_df_match": "02_annotate.ipynb",
         "load_df": "02_annotate.ipynb",
         "load_completions": "02_annotate.ipynb",
         "get_og_filepath": "02_annotate.ipynb",
         "anno_sample_merge": "02_annotate.ipynb",
         "nnAnnotations": "02_annotate.ipynb",
         "nnPredict": "03_inference.ipynb",
         "predict_from_sample_df": "03_inference.ipynb",
         "predict_sample": "03_inference.ipynb",
         "predict": "03_inference.ipynb"}

modules = ["core.py",
           "sample.py",
           "annotate.py",
           "inference.py"]

doc_url = "https://Living-with-machines.github.io/nnanno/"

git_url = "https://github.com/Living-with-machines/nnanno/tree/master/"

def custom_doc_links(name): return None
